require 'json'
require 'faraday'

module Blobstache
  class Client
    VERSION = '0.0.2'
    attr_accessor :host, :port, :user_id, :key

    def initialize(host: 'localhost', port: 1234, user_id: 'id', key: "key")
      self.host = host
      self.port = port
      self.user_id = user_id
      self.key = key
    end

# UU   UU  SSSSS  EEEEEEE RRRRRR  
# UU   UU SS      EE      RR   RR 
# UU   UU  SSSSS  EEEEE   RRRRRR  
# UU   UU      SS EE      RR  RR  
#  UUUUU   SSSSS  EEEEEEE RR   RR 
                                
    def create_user()
      respond(connection.post("users"))
    end

    def list_users
      respond(connection.get("users"))
    end

    def delete_user(id)
      respond(connection.delete("users/#{id}"))
    end

# BBBBB   UU   UU  CCCCC  KK  KK EEEEEEE TTTTTTT 
# BB   B  UU   UU CC    C KK KK  EE        TTT   
# BBBBBB  UU   UU CC      KKKK   EEEEE     TTT   
# BB   BB UU   UU CC    C KK KK  EE        TTT   
# BBBBBB   UUUUU   CCCCC  KK  KK EEEEEEE   TTT   
                                               
    def create_bucket(name)
      respond(connection.post("buckets") do |req|
        req.headers['Bucketid'] = name
      end
      )
    end

    def list_buckets
      respond(connection.get("buckets"))
    end

    def get_bucket(id)
      respond(connection.get("buckets/#{id}"))
    end

    def delete_bucket(id)
      respond(connection.delete("buckets/#{id}"))
    end

#  OOOOO  BBBBB       JJJ EEEEEEE  CCCCC  TTTTTTT 
# OO   OO BB   B      JJJ EE      CC    C   TTT   
# OO   OO BBBBBB      JJJ EEEEE   CC        TTT   
# OO   OO BB   BB JJ  JJJ EE      CC    C   TTT   
#  OOOO0  BBBBBB   JJJJJ  EEEEEEE  CCCCC    TTT   

    def create_object(bucket, name, body = "")
      respond(connection.post("objects") do |req|
        req.headers['Bucketid'] = bucket
        req.headers['Objectalias'] = name
        req.body = body
      end)
    end

    def list_objects(bucket)
      respond(connection.get("objects") do |req|
        req.headers['Bucketid'] = bucket
      end)
    end

    def set_object_public(bucket, id)
      respond(connection.put("public/objects/#{id}") do |req|
        req.headers['Bucketid'] = bucket
      end)
    end

    def get_object_info(bucket, id)
      respond(connection.get("info/objects/#{id}") do |req|
        req.headers['Bucketid'] = bucket
      end)
    end

    def get_object_size(bucket, id)
      r = connection.head("objects/#{id}") do |req|
        req.headers['Bucketid'] = bucket
      end
      r.headers["Object-Size"].to_i
    end

    def get_object(bucket, id)
      connection.get("objects/#{id}") do |req|
        req.headers['Bucketid'] = bucket
      end
    end

    def delete_object(bucket, id)
      respond(connection.delete("objects/#{id}") do |req|
        req.headers['Bucketid'] = bucket
      end)
    end

    private

    def respond(response)
      if response.status / 100 == 2 and response.body != nil and response.body != ""
        return JSON.parse(response.body)
      end
      response
    rescue
      response
    end

    def connection
      @connection ||= ::Faraday.new(
        url: "http://#{host}:#{port}", 
        ssl: {:verify => false},
        headers: {
          "Userid" => user_id,
          "Key" => key
        }
      )
    end

  end
end